package eu.jonahbauer.raytracing.render.spectral.spectrum;

import eu.jonahbauer.raytracing.render.spectral.colors.Chromaticity;
import org.jetbrains.annotations.NotNull;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UncheckedIOException;
import java.nio.charset.StandardCharsets;

public final class Spectra {
    private static final Spectrum X;
    private static final Spectrum Y;
    private static final Spectrum Z;

    static {
        var count = Spectrum.LAMBDA_MAX - Spectrum.LAMBDA_MIN + 1;
        var x = new double[count];
        var y = new double[count];
        var z = new double[count];

        try (
                var is = Spectra.class.getResourceAsStream("/eu/jonahbauer/raytracing/spectrum/CIE_xyz_1931_2deg.csv");
                var in = new BufferedReader(new InputStreamReader(is, StandardCharsets.US_ASCII))
        ) {
            for (int i = 0; i < count; i++) {
                var line = in.readLine().split(",");
                x[i] = Double.parseDouble(line[1]);
                y[i] = Double.parseDouble(line[2]);
                z[i] = Double.parseDouble(line[3]);
            }
        } catch (IOException e) {
            throw new UncheckedIOException(e);
        }

        X = new DenselySampledSpectrum(x, Spectrum.LAMBDA_MIN);
        Y = new DenselySampledSpectrum(y, Spectrum.LAMBDA_MIN);
        Z = new DenselySampledSpectrum(z, Spectrum.LAMBDA_MIN);
    }

    /**
     * {@return the X color matching curve}
     */
    public static @NotNull Spectrum X() {
        return X;
    }

    /**
     * {@return the Y color matching curve}
     */
    public static @NotNull Spectrum Y() {
        return Y;
    }

    /**
     * {@return the Z color matching curve}
     */
    public static @NotNull Spectrum Z() {
        return Z;
    }

    /**
     * {@return the D illuminant at the given temperature}
     * @param T the temperature
     */
    public static @NotNull Spectrum D(double T) {
        // determine chromaticity
        double x;
        if (4000 <= T && T <= 7000) {
            x = 0.244063 + 0.09911E3 / T + 2.9678E6 / (T * T) - 4.6070E9 / (T * T * T);
        } else if (7000 < T && T <= 25000) {
            x = 0.237040 + 0.24748E3 / T + 1.9018E6 / (T * T) - 2.0064E9 / (T * T * T);
        } else {
            throw new IllegalArgumentException("T must be between 4000 and 25000, inclusive");
        }
        var y = Math.fma(Math.fma(-3.000, x, 2.870), x, -0.275);

        var m = 0.0241 + 0.2562 * x - 0.7341 * y;
        var m1 = (-1.3515 -  1.7703 * x +  5.9114 * y) / m;
        var m2 = ( 0.0300 - 31.4424 * x + 30.0717 * y) / m;


    }

    public static @NotNull Spectrum getNamed(@NotNull String name) {

    }

    private Spectra() {
        throw new UnsupportedOperationException();
    }
}
